(ns com.oly.static-sites.do-blog.pages.chess.movement)

(defn line-horizontal [[x y] size]
  (loop [cords [[1 y]]]
    (if (every? #(< % size) (-> cords last))
      (recur (conj cords [(+ 1 (-> cords last first)) y]))
      cords)))

(defn line-vertical [[x y] size]
  (loop [cords [[x 1]]]
    (if (every? #(< % size) (-> cords last))
      (recur (conj cords [x (+ 1 (-> cords last last))]))
      cords)))

(defn line-projection [[x y] size]
  (concat (line-horizontal [x y] size)
          (line-vertical [x y] size)))

(defn diagonal-start-ascending [x y]
  [(+ 1 (- x (min x y)))
   (+ 1 (- y (min x y)))])

(defn diagonal-start-descending [x y]
  [(+ x (- x y))
   (- y (- x y))])

(defn diagonal-ascending
  "down and to the right"
  [[x y] size]
  (let [lowest (min (- 8 x) (- 8 y))
        highest (max x y)
        start (diagonal-start-ascending x y)]
    (loop [step lowest
           cords [start]]
      (if  (or (< step 8)
               (every? #(< % 8) (-> cords last)))
        (recur (+ step 1)
               (conj cords [(+ (-> cords last first) 1)
                            (+ (-> cords last last) 1)]))
        cords))))

(defn diagonal-descending [[x y] size]
  (let [highest (max x y)
        start (diagonal-start-descending x y)]
    (loop [step (+ 1 highest)
           cords [start]]
      (if  (or (> step 1)
               (every? #(< % 1) (-> cords last)))
        (recur (- step 1)
               (conj cords [(+ (-> cords last first) 1)
                            (- (-> cords last last) 1)]))
        cords))))

(defn diagonal [xy size direction]
  (if (= direction :desc)
    (diagonal-ascending xy size)
    (diagonal-descending xy size)))

(defn diagonal-projection [[x y] size]
  (concat (diagonal-ascending [x y] size)
          (diagonal-descending [x y] size)))

(defn directional-projection [[x y] size]
  (concat (diagonal-projection [x y] size)
          (line-projection [x y] 8)))

(defn star [[x y] size]
  (loop [s 1
         cords [[x y]]]
    (case s
      3
      (recur (+ s 1)
             (conj cords
                   ;[(- (-> cords first first) s) (-> cords first last)]
                   [(- (-> cords first first) 2)  (+ (-> cords first last) 1)]
                   [(- (-> cords first first) 2)  (- (-> cords first last) 1)]

                   ;[(+ (-> cords first first) s) (-> cords first last)]
                   [(+ (-> cords first first) 2)  (- (-> cords first last) 1)]
                   [(+ (-> cords first first) 2)  (+ (-> cords first last) 1)]

                   [(+ (-> cords first first) 1) (- (-> cords first last) 2)]
                   [(- (-> cords first first) 1) (- (-> cords first last) 2)]

                   [(- (-> cords first first) 1) (+ (-> cords first last) 2)]
                   [(+ (-> cords first first) 1) (+ (-> cords first last) 2)]))
      4 cords
      (recur (+ s 1)
             (conj cords
                   [(- (-> cords first first) s) (-> cords first last)]
                   [(+ (-> cords first first) s) (-> cords first last)]
                   [(-> cords first first)  (- (-> cords first last) s)]
                   [(-> cords first first)  (+ (-> cords first last) s)])))))

(defn all-around-projection [[x y] size]
  [[(- x 1) (+ 1 y)] [x (+ 1 y)] [(+ 1 x) (+ 1 y)]
   [(- x 1) y] [(+ 1 x) y]
   [(- x 1) (- y 1)] [x (- y 1)] [(+ 1 x) (- y 1)]])

(defn pawn-projection [[x y] direction size]
  (if (or (= y 2) (= y 7))
    [[x (+ y direction)] [x (+ y (* 2 direction))]]
    [[x (+ y direction)]]))

(defn fetch-selected-piece-moves [piece]
  (case (:type piece)
    :king (all-around-projection (:start-pos piece) 8)
    :queen (directional-projection (:start-pos piece) 8)
    :rook (line-projection (:start-pos piece) 8)
    :knight (star (:start-pos piece) 8)
    :pawn (pawn-projection (:start-pos piece) (if (= :white (:colour piece)) 1 -1) 8)
    :bishop (diagonal-projection (:start-pos piece) 8)
    (prn (str "type = " (:type piece)))))

;; (deftest line-projection-test
;;   (testing "Horizontal direction"
;;     (is (= (line-horizontal [2 3] 8)
;;            [[1 3] [2 3] [3 3] [4 3] [5 3] [6 3] [7 3] [8 3]])))
;;   (testing "Vertical direction"
;;     (is (= (line-vertical [2 3] 8)
;;            [[2 1] [2 2] [2 3] [2 4] [2 5] [2 6] [2 7] [2 8]]))))

;; (deftest diagonal-projection-test
;;   (testing "Diagonal ascending direction"
;;     (is (= (diagonal-ascending [2 3] 8)
;;            [[1 2] [2 3] [3 4] [4 5] [5 6] [6 7] [7 8]])))
;;   (testing "Diagonal descending direction"
;;     (is (= (diagonal-descending [2 3] 8)
;;            [[8 7] [7 6] [6 5] [5 4] [4 3] [3 2] [2 1]])))

;;   (testing "Diagonals "
;;     (is (= (diagonal-projection [2 3] 8)
;;            [[1 2] [2 3] [3 4] [4 5] [5 6] [6 7] [7 8]]))))
